@@ -9,6 +9,7 @@ import logging
 import os
 import shutil
 import subprocess
+import sys
 import tempfile
 import textwrap
 from itertools import chain
@@ -18,7 +19,7 @@ from typing import ContextManager, Dict, List, NamedTuple, Optional, Set, Tuple
 
 from . import buck, json_rpc, log, statistics
 from .buck import BuckBuilder, find_buck_root
-from .configuration import Configuration
+from .configuration import Configuration, SearchPathElement, SimpleSearchPathElement
 from .exceptions import EnvironmentException
 from .filesystem import (
     _compute_symbolic_link_mapping,
@@ -106,7 +107,7 @@ class UpdatedPaths(NamedTuple):
 class AnalysisDirectory:
     def __init__(
         self,
-        path: str,
+        path: SearchPathElement,
         filter_paths: Optional[Set[str]] = None,
         search_path: Optional[List[str]] = None,
     ) -> None:
@@ -116,8 +117,11 @@ class AnalysisDirectory:
             search_path or []
         )
 
+    def get_command_line_root(self) -> str:
+        return os.path.abspath(self._path.command_line_argument())
+
     def get_root(self) -> str:
-        return self._path
+        return self._path.path()
 
     def get_filter_roots(self) -> Set[str]:
         current_project_directories = self._filter_paths or {self.get_root()}
@@ -186,7 +190,7 @@ class AnalysisDirectory:
 class SharedAnalysisDirectory(AnalysisDirectory):
     def __init__(
         self,
-        source_directories: List[str],
+        source_directories: List[SearchPathElement],
         targets: List[str],
         project_root: str,
         original_directory: Optional[str] = None,
@@ -199,7 +203,7 @@ class SharedAnalysisDirectory(AnalysisDirectory):
         configuration: Optional[Configuration] = None,
         temporary_directories: Optional[List[str]] = None,
     ) -> None:
-        self._source_directories: Set[str] = set(source_directories)
+        self._source_directories: Set[SearchPathElement] = set(source_directories)
         self._targets: List[str] = targets
         self._original_directory = original_directory
         self._project_root = project_root
@@ -231,13 +235,17 @@ class SharedAnalysisDirectory(AnalysisDirectory):
             return os.path.join(self._project_root, ".pyre")
 
     @functools.lru_cache(1)
-    def get_root(self) -> str:
+    def get_command_line_root(self) -> str:
         path_to_root = self._local_configuration_root or "shared_analysis_directory"
         suffix = "_{}".format(str(os.getpid())) if self._isolate else ""
         return os.path.join(
             self.get_scratch_directory(), "{}{}".format(path_to_root, suffix)
         )
 
+    @functools.lru_cache(1)
+    def get_root(self) -> str:
+        return self.get_command_line_root()
+
     def get_filter_roots(self) -> Set[str]:
         current_project_directories = self._filter_paths or {self._project_root}
         if len(self._targets) == 0:
@@ -278,6 +286,7 @@ class SharedAnalysisDirectory(AnalysisDirectory):
                 "number_of_unsupported_files": number_of_unsupported_files,
             },
             normals={
+                "command_line": " ".join(sys.argv),
                 "event_type": event_type.value,
                 "local_root": self._local_configuration_root,
                 "buck_builder_type": str(self._buck_builder),
@@ -298,7 +307,9 @@ class SharedAnalysisDirectory(AnalysisDirectory):
                     new_source_directories,
                     original_directory,
                 )
-            self._source_directories.update(new_source_directories)
+            self._source_directories.update(
+                (SimpleSearchPathElement(path) for path in new_source_directories)
+            )
 
         if len(self._source_directories) == 0:
             message = """
@@ -351,7 +362,7 @@ class SharedAnalysisDirectory(AnalysisDirectory):
         ):
             all_paths = {}
             for source_directory in self._source_directories:
-                self._merge_into_paths(source_directory, all_paths)
+                self._merge_into_paths(source_directory.path(), all_paths)
             for relative_path, project_path in all_paths.items():
                 scratch_path = os.path.join(root, relative_path)
                 if os.path.realpath(scratch_path) != project_path:
@@ -400,16 +411,18 @@ class SharedAnalysisDirectory(AnalysisDirectory):
             message_type = json_rpc.LanguageServerMessageType.INFORMATION.value
         show_status_message = json_rpc.Request(
             method="window/showStatus",
-            parameters={
-                "message": message,
-                "shortMessage": short_message,
-                "type": message_type,
-                "actions": [],
-                "progress": {
-                    "numerator": DONT_CARE_PROGRESS_VALUE,
-                    "denominator": DONT_CARE_PROGRESS_VALUE,
-                },
-            },
+            parameters=json_rpc.ByNameParameters(
+                {
+                    "message": message,
+                    "shortMessage": short_message,
+                    "type": message_type,
+                    "actions": [],
+                    "progress": {
+                        "numerator": DONT_CARE_PROGRESS_VALUE,
+                        "denominator": DONT_CARE_PROGRESS_VALUE,
+                    },
+                }
+            ),
         )
         try:
             with SocketConnection(configuration.log_directory) as socket_connection:
@@ -491,6 +504,7 @@ class SharedAnalysisDirectory(AnalysisDirectory):
         LOG.info("Updating shared directory `%s`", self.get_root())
 
         self.rebuild()
+        # pyre-fixme[35]: Target cannot be annotated.
         new_paths: Set[str] = set(self._symbolic_links.keys())
 
         buck.clear_buck_query_cache()
@@ -695,7 +709,7 @@ class SharedAnalysisDirectory(AnalysisDirectory):
 
         all_paths = {}
         for source_directory in self._source_directories:
-            self._merge_into_paths(source_directory, all_paths)
+            self._merge_into_paths(source_directory.path(), all_paths)
         for relative, original in all_paths.items():
             merged = os.path.join(root, relative)
             add_symbolic_link(merged, original)
@@ -816,10 +830,16 @@ def resolve_analysis_directory(
 
     # Only read from the configuration if no explicit targets are passed in.
     if not source_directories and not targets:
-        source_directories = list(configuration.source_directories)
+        source_paths: List[
+            SearchPathElement
+        ] = configuration.get_existent_source_directories()
         targets = list(configuration.targets)
     else:
-        source_directories = source_directories or []
+        source_paths: List[SearchPathElement] = [
+            # TODO: support SubdirectorySearchPathElement here too?
+            SimpleSearchPathElement(path)
+            for path in source_directories
+        ]
         targets = targets or []
         if targets:
             configuration_name = LOCAL_CONFIGURATION_FILE
@@ -839,9 +859,9 @@ def resolve_analysis_directory(
             local_configuration_root, project_root
         )
 
-    if len(source_directories) == 1 and len(targets) == 0:
+    if len(source_paths) == 1 and len(targets) == 0:
         analysis_directory = AnalysisDirectory(
-            source_directories.pop(),
+            source_paths.pop(),
             filter_paths=filter_paths,
             search_path=[
                 search_path.path()
@@ -854,14 +874,14 @@ def resolve_analysis_directory(
         )
 
         analysis_directory = SharedAnalysisDirectory(
-            source_directories=source_directories,
+            source_directories=source_paths,
             targets=targets,
             buck_builder=buck_builder,
             original_directory=original_directory,
             project_root=project_root,
             filter_paths=filter_paths,
             local_configuration_root=local_configuration_root,
-            extensions=configuration.get_valid_extensions(),
+            extensions=configuration.get_valid_extension_suffixes(),
             search_path=[
                 search_path.path()
                 for search_path in configuration.get_existent_search_paths()
@@ -870,4 +890,5 @@ def resolve_analysis_directory(
             configuration=configuration,
             temporary_directories=temporary_directories,
         )
+
     return analysis_directory
