@@ -71,7 +71,10 @@ class Check(Reporting):
         excludes = self._configuration.excludes
         for exclude in excludes:
             flags.extend(["-exclude", exclude])
-        extensions = self._configuration.get_valid_extensions()
+        extensions = [
+            extension.command_line_argument()
+            for extension in self._configuration.extensions
+        ]
         for extension in extensions:
             flags.extend(["-extension", extension])
         return flags
