@@ -93,16 +93,14 @@ class Result:
 
 
 def _convert_json_response_to_result(response: json_rpc.Response) -> Result:
-    error = response.error
-    error_message = None
-    if error:
-        error_code = ExitCode.FAILURE
-        error_message = json.dumps(error)
+    if isinstance(response, json_rpc.ErrorResponse):
+        return Result(output="", code=ExitCode.FAILURE, error=response.serialize())
+    elif isinstance(response, json_rpc.SuccessResponse):
+        return Result(
+            output=json.dumps(response.result), code=ExitCode.SUCCESS, error=None
+        )
     else:
-        error_code = ExitCode.SUCCESS
-    return Result(
-        output=json.dumps(response.result), code=error_code, error=error_message
-    )
+        raise RuntimeError("Response should be either success or error.")
 
 
 def executable_file(file_path: str) -> str:
@@ -228,7 +226,7 @@ class Command(CommandParser, ABC):
         if self._show_error_traces:
             flags.append("-show-error-traces")
         logging_sections = self._logging_sections
-        if not self._capable_terminal:
+        if not self._capable_terminal or self._noninteractive:
             # Disable progress reporting for non-capable terminals.
             # This helps in reducing clutter.
             if logging_sections:
@@ -291,7 +289,7 @@ class Command(CommandParser, ABC):
             raise InvalidConfiguration("Cannot find any pyre binary.")
         client_command = [binary, command]
         client_command.extend(self._flags())
-        client_command.append(self._analysis_directory.get_root())
+        client_command.append(self._analysis_directory.get_command_line_root())
 
         def limit_memory_usage() -> None:
             try:
