@@ -6,7 +6,7 @@
 
 import json
 from pathlib import Path
-from typing import Any, Dict, List, Optional, Set, Type, Union
+from typing import Any, Dict, List, Mapping, Optional, Set, Type, Union
 
 import libcst as cst
 from libcst._exceptions import ParserSyntaxError
@@ -59,7 +59,7 @@ def _parse_paths(paths: List[Path]) -> List[Path]:
 
 
 def _path_wise_counts(
-    paths: Dict[str, Union[cst.Module, cst.MetadataWrapper]],
+    paths: Mapping[str, Union[cst.Module, cst.MetadataWrapper]],
     collector_class: Type[StatisticsCollector],
     strict: bool = False,
 ) -> Dict[str, StatisticsCollector]:
@@ -107,26 +107,14 @@ class Statistics(Command):
         self._filter_paths: Set[str] = set(filter_paths)
         self._log_results: bool = log_results
 
-    def _collect_statistics(self, modules: Dict[str, cst.Module]) -> Dict[str, Any]:
-        modules_with_metadata = {
+    def _collect_statistics(self, modules: Mapping[str, cst.Module]) -> Dict[str, Any]:
+        modules_with_metadata: Mapping[str, cst.MetadataWrapper] = {
             path: MetadataWrapper(module) for path, module in modules.items()
         }
-        # pyre-fixme[6]: Expected `Dict[str, Union[cst._nodes.module.Module,
-        #  cst.metadata.wrapper.MetadataWrapper]]` for 1st param but got `Dict[str,
-        #  cst.metadata.wrapper.MetadataWrapper]`.
         annotations = _path_wise_counts(modules_with_metadata, AnnotationCountCollector)
-        # pyre-fixme[6]: Expected `Dict[str, Union[cst._nodes.module.Module,
-        #  cst.metadata.wrapper.MetadataWrapper]]` for 1st param but got `Dict[str,
-        #  cst._nodes.module.Module]`.
         fixmes = _path_wise_counts(modules, FixmeCountCollector)
-        # pyre-fixme[6]: Expected `Dict[str, Union[cst._nodes.module.Module,
-        #  cst.metadata.wrapper.MetadataWrapper]]` for 1st param but got `Dict[str,
-        #  cst._nodes.module.Module]`.
         ignores = _path_wise_counts(modules, IgnoreCountCollector)
         strict_files = _path_wise_counts(
-            # pyre-fixme[6]: Expected `Dict[str, Union[cst._nodes.module.Module,
-            #  cst.metadata.wrapper.MetadataWrapper]]` for 1st param but got `Dict[str,
-            #  cst._nodes.module.Module]`.
             modules,
             StrictCountCollector,
             self._configuration.strict,
