@@ -11,12 +11,16 @@ import logging
 import logging.handlers
 import os
 import re
+import subprocess
 import sys
 import threading
 import time
+from pathlib import Path
 from types import TracebackType
 from typing import Generator, Iterable, Optional, Pattern, Sequence
 
+import click
+
 
 PERFORMANCE: int = 15
 PROMPT: int = 50
@@ -82,11 +86,13 @@ class TimedStreamHandler(logging.StreamHandler):
         self._active_lines: int = 0
 
         # Preamble preparing terminal.
-        sys.stderr.write(
+        click.echo(
             Format.NEWLINE
             + Format.CLEAR_LINE
             + Format.CURSOR_UP_LINE
-            + Format.HIDE_CURSOR
+            + Format.HIDE_CURSOR,
+            file=sys.stderr,
+            nl=False,
         )
 
         thread = threading.Thread(target=self._thread)
@@ -105,24 +111,25 @@ class TimedStreamHandler(logging.StreamHandler):
 
     def emit(self, record: logging.LogRecord, age: Optional[float] = None) -> None:
         suffix = ""
-        color = ""
-        active_lines = record.msg.count("\n") + 1
+        color: Optional[str] = None
+        message = record.msg
+        active_lines = message.count("\n") + 1
         truncate = Format.TRUNCATE_OVERFLOW
         if record.levelname in self.LINE_BREAKING_LEVELS:
-            record.msg += "\n"
+            message += "\n"
 
         if record.levelname == "ERROR":
-            color = Color.RED
+            color = "red"
             self._record = None
             active_lines = 0
             truncate = Format.WRAP_OVERFLOW
         elif record.levelname == "WARNING":
-            color = Color.YELLOW
+            color = "yellow"
             self._record = None
             active_lines = 0
             truncate = Format.WRAP_OVERFLOW
         elif record.levelname == "PROMPT":
-            color = Color.YELLOW
+            color = "yellow"
             self._record = None
             active_lines = 0
             truncate = Format.WRAP_OVERFLOW
@@ -132,27 +139,23 @@ class TimedStreamHandler(logging.StreamHandler):
             truncate = Format.WRAP_OVERFLOW
         elif age:
             if age > 10:
-                color = Color.YELLOW
+                color = "yellow"
             if age > 30:
-                color = Color.RED
-            suffix = " {}[{:.1f}s]{}".format(
-                color if color else "", age, Format.CLEAR if color else ""
-            )
+                color = "red"
+            suffix = click.style(" [{:.1f}s]".format(age), fg=color)
         else:
             self._record = record
             self._last_update = time.time()
 
+        prompt = click.style(f"{Character.LAMBDA}", fg=color)
+        new_message = f"{self.clear_lines()}{prompt} {truncate}{message}{suffix}"
+
         timed_record = copy.copy(record)
         timed_record.msg = (
-            "{clear_line}{color} {cursor}{clear} {truncate}{message}{suffix}"
-        ).format(
-            clear_line=self.clear_lines(),
-            color=color,
-            cursor=Character.LAMBDA,
-            clear=Format.CLEAR,
-            truncate=truncate,
-            message=record.msg,
-            suffix=suffix,
+            f"{click.unstyle(new_message)}\n"
+            # pyre-ignore[16]: Missing typeshed stub for this API
+            if click.utils.should_strip_ansi(stream=sys.stderr)
+            else new_message
         )
         self._active_lines = active_lines
         super(TimedStreamHandler, self).emit(timed_record)
@@ -170,11 +173,11 @@ class TimedStreamHandler(logging.StreamHandler):
         self._terminate = True
 
         if self._active_lines > 0:
-            sys.stderr.write(self.clear_lines())
+            click.echo(self.clear_lines(), file=sys.stderr, nl=False)
             self._active_lines = 0
 
         # Reset terminal.
-        sys.stderr.write(Format.WRAP_OVERFLOW + Format.SHOW_CURSOR)
+        click.echo(Format.WRAP_OVERFLOW + Format.SHOW_CURSOR, file=sys.stderr, nl=False)
         sys.stderr.flush()
 
 
@@ -227,9 +230,9 @@ def cleanup() -> None:
 
     output = stdout.getvalue()
     if output:
-        sys.stdout.write(output)
+        click.echo(output, nl=False)
         if not output.endswith("\n"):
-            sys.stdout.write("\n")
+            click.echo()
 
 
 @contextlib.contextmanager
@@ -241,6 +244,45 @@ def configured_logger(noninteractive: bool) -> Generator[None, None, None]:
         cleanup()
 
 
+@contextlib.contextmanager
+def file_tailer(file_path: Path) -> Generator[Iterable[str], None, None]:
+    """
+    This function yields a stream of string generated by following the last
+    part of the given file. In other words, the returned stream behaves roughtly
+    the same as `tail -F`: If the file being watched is left untouched, invoking
+    `next` on the returned stream will block indefinitely. If the file being
+    watched gets a line appended to the end of it, invoking `next` on the returned
+    stream will return the appended line. Leaving the context manager will cause
+    the returned stream to stop iteration next time `next` is invoked.
+
+    This API is intended to be used along with `StreamLogger` to concurrently
+    forward the content of a log file to the terminal in the background:
+
+    ```
+    with file_tailer(log_file) as log_stream:
+        with StreamLogger(log_stream) as logger:
+            # Main thread logic happens here
+            ...
+    logger.join()
+    ```
+    """
+    with subprocess.Popen(
+        ["tail", "-F", "-n", "0", str(file_path)],
+        stdout=subprocess.PIPE,
+        stderr=subprocess.DEVNULL,
+        universal_newlines=True,
+    ) as tail:
+        try:
+            stdout = tail.stdout
+            if stdout is None:
+                raise RuntimeError(
+                    "subprocess.Popen failed to set up a pipe for stdout"
+                )
+            yield stdout
+        finally:
+            tail.terminate()
+
+
 class StreamLogger:
     _should_stop_reading_stream = False
     _current_section: Optional[str]
