@@ -42,10 +42,9 @@ def _log_statistics(
     client_exception_message: str,
     error_message: Optional[str],
     exit_code: int,
-    should_log: bool = True,
 ) -> None:
     configuration = command.configuration
-    if should_log and configuration and configuration.logger:
+    if configuration and configuration.logger:
         statistics_module.log_with_configuration(
             category=statistics_module.LoggerCategory.USAGE,
             configuration=configuration,
@@ -58,6 +57,7 @@ def _log_statistics(
                 "root": configuration.relative_local_root,
                 "cwd": os.getcwd(),
                 "client_version": __version__,
+                "command_line": " ".join(sys.argv),
                 "command": command.NAME,
                 "client_exception": client_exception_message,
                 "error_message": error_message,
@@ -105,23 +105,24 @@ def run_pyre_command(
     start_time = time.time()
 
     client_exception_message = ""
-    should_log_statistics = True
     # Having this as a fails-by-default helps flag unexpected exit
     # from exception flows.
     exit_code = ExitCode.FAILURE
     try:
         configuration_module.check_nested_local_configuration(configuration)
+        configuration_module.check_open_source_version(configuration)
         log.start_logging_to_directory(noninteractive, configuration.log_directory)
+        LOG.debug(f"Running cli command `{' '.join(sys.argv)}`...")
         exit_code = command.run().exit_code()
     except (buck.BuckException, EnvironmentException) as error:
         client_exception_message = str(error)
         exit_code = ExitCode.FAILURE
         if isinstance(error, buck.BuckException):
             exit_code = ExitCode.BUCK_ERROR
-    except (
-        commands.ClientException,
-        configuration_module.InvalidConfiguration,
-    ) as error:
+    except (configuration_module.InvalidConfiguration) as error:
+        client_exception_message = str(error)
+        exit_code = ExitCode.CONFIGURATION_ERROR
+    except commands.ClientException as error:
         client_exception_message = str(error)
         exit_code = ExitCode.FAILURE
     except Exception:
@@ -143,7 +144,6 @@ def run_pyre_command(
             client_exception_message,
             error_message,
             exit_code,
-            should_log_statistics,
         )
     return exit_code
 
@@ -252,7 +252,7 @@ def _create_configuration_with_retry(
             "Cannot determine which recent local root to rerun. "
         )
 
-    LOG.warning(f"Restarting pyre under local root `{local_root_for_rerun}`...")
+    LOG.warning(f"Running pyre under local root `{local_root_for_rerun}`...")
     LOG.warning(
         f"Hint: To avoid this prompt, run `pyre -l {local_root_for_rerun}` "
         f"or `cd {local_root_for_rerun} && pyre`."
@@ -769,16 +769,38 @@ def persistent(context: click.Context, no_watchman: bool) -> int:
     configuration = configuration_module.create_configuration(
         command_argument, Path(".")
     )
-    return run_pyre_command(
-        commands.Persistent(
-            command_argument,
-            original_directory=os.getcwd(),
-            configuration=configuration,
-            no_watchman=no_watchman,
-        ),
-        configuration,
-        True,
-    )
+    if command_argument.use_command_v2:
+        log.start_logging_to_directory(
+            command_argument.noninteractive, configuration.log_directory
+        )
+        return v2.persistent.run(
+            configuration,
+            command_arguments.StartArguments(
+                changed_files_path=command_argument.changed_files_path,
+                debug=command_argument.debug,
+                load_initial_state_from=command_argument.load_initial_state_from,
+                no_saved_state=command_argument.no_saved_state,
+                no_watchman=no_watchman,
+                save_initial_state_to=command_argument.save_initial_state_to,
+                saved_state_project=command_argument.saved_state_project,
+                sequential=command_argument.sequential,
+                show_error_traces=command_argument.show_error_traces,
+                store_type_check_resolution=False,
+                terminal=False,
+                wait_on_initialization=True,
+            ),
+        )
+    else:
+        return run_pyre_command(
+            commands.Persistent(
+                command_argument,
+                original_directory=os.getcwd(),
+                configuration=configuration,
+                no_watchman=no_watchman,
+            ),
+            configuration,
+            True,
+        )
 
 
 @pyre.command()
@@ -828,16 +850,20 @@ def query(context: click.Context, query: str) -> int:
     """
     command_argument: command_arguments.CommandArguments = context.obj["arguments"]
     configuration = _create_configuration_with_retry(command_argument, Path("."))
-    return run_pyre_command(
-        commands.Query(
-            command_argument,
-            original_directory=os.getcwd(),
-            configuration=configuration,
-            query=query,
-        ),
-        configuration,
-        command_argument.noninteractive,
-    )
+
+    if command_argument.use_command_v2:
+        return v2.query.run(configuration, query)
+    else:
+        return run_pyre_command(
+            commands.Query(
+                command_argument,
+                original_directory=os.getcwd(),
+                configuration=configuration,
+                query=query,
+            ),
+            configuration,
+            command_argument.noninteractive,
+        )
 
 
 @pyre.command()
@@ -1116,6 +1142,29 @@ def stop(context: click.Context) -> int:
         )
 
 
+@pyre.command()
+@click.pass_context
+def validate_models(context: click.Context) -> int:
+    """
+    Validate the taint models for the given project by querying the Pyre server.
+    """
+    command_argument: command_arguments.CommandArguments = context.obj["arguments"]
+    configuration = _create_configuration_with_retry(command_argument, Path("."))
+
+    if command_argument.use_command_v2:
+        return v2.validate_models.run(configuration, output=command_argument.output)
+    else:
+        return run_pyre_command(
+            commands.ValidateModels(
+                command_argument,
+                original_directory=os.getcwd(),
+                configuration=configuration,
+            ),
+            configuration,
+            command_argument.noninteractive,
+        )
+
+
 # Need the default argument here since this is our entry point in setup.py
 def main(argv: List[str] = sys.argv[1:]) -> int:
     noninteractive = ("-n" in argv) or ("--noninteractive" in argv)
